/*
** Module   : NumberedException.java
** Abstract : Exception type which supports a queryable error number
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------------------------------
** 001 ECF 20051104 @23243   Created initial version. Exception type which
**                           supports a queryable error number in addition
**                           to regular Exception features.
** 002 SVL 20140709          Added prefix field.
** 003 VIG 20140709          Changed field number to numbers and added 
**                           additional constructor and getter for it.
*/ 

package com.goldencode.p2j;

import org.apache.commons.lang.*;

/**
 * Exception type which supports a queryable error number. The number may
 * be set only upon construction and can be queried using the {@link
 * #getNumber} method.  The default error number is <code>0</code>, in case
 * a constructor variant is used which does not accept an error number.
 * These are provided primarily for compatibility with the superclass, so
 * that subclasses need not expressly set an error number.
 */
public class NumberedException
extends Exception
{
   /** Errors numbers in order of appearance */
   private int[] numbers;

   /** Determines if the prefix should be displayed for the error message. */
   private boolean prefix = true;
   
   /**
    * Constructor which sets the message text only.  The default error
    * number is used.
    *
    * @param   message
    *          Error message. Corresponds to the last error number in the list. 
    */
   public NumberedException(String message)
   {
      super(message);
   }
   
   /**
    * Constructor which sets message text and error number.
    *
    * @param   message
    *          Error message.
    * @param   number
    *          Error number.
    */
   public NumberedException(String message, int number)
   {
      this(message, number, (Throwable)null);
   }
   
   /**
    * Constructor which sets message text and errors numbers.
    *
    * @param   message
    *          Error message.
    * @param   numbers
    *          Contains numbers of errors with previous during one statement execution.
    */
   public NumberedException(String message, int[] numbers)
   {
      super(message, null);
      this.numbers = new int[0];
      this.numbers = ArrayUtils.addAll(this.numbers, numbers);
   }

   /**
    * Constructor which sets message text and error number.
    *
    * @param   message
    *          Error message.
    * @param   number
    *          Error number.
    * @param   prefix
    *          Determines if the prefix should be displayed for the error message.
    */
   public NumberedException(String message, int number, boolean prefix)
   {
      this(message, number, prefix, null);
   }
   
   /**
    * Constructor which sets message text and error number, and which chains
    * a root cause <code>Throwable</code>.
    *
    * @param   message
    *          Error message.
    * @param   number
    *          Error number.
    * @param   rootCause
    *          Root cause exception/error.
    */
   public NumberedException(String message, int number, Throwable rootCause)
   {
      super(message, rootCause);
      this.numbers = new int[1];
      this.numbers[0] = number;
   }

   /**
    * Constructor which sets message text and error number, and which chains
    * a root cause <code>Throwable</code>.
    *
    * @param   message
    *          Error message.
    * @param   number
    *          Error number.
    * @param   prefix
    *          Determines if the prefix should be displayed for the error message.
    * @param   rootCause
    *          Root cause exception/error.
    */
   public NumberedException(String message, int number, boolean prefix, Throwable rootCause)
   {
      super(message, rootCause);
      this.numbers = new int[1];
      this.numbers[0] = number;
      this.prefix = prefix;
   }

   /**
    * Constructor which sets message text and which chains a root cause
    * <code>Throwable</code>.  The default error number is used.
    *
    * @param   message
    *          Error message.
    * @param   rootCause
    *          Root cause exception/error.
    */
   public NumberedException(String message, Throwable rootCause)
   {
      super(message, rootCause);
   }
   
   /**
    * Constructor which chains a root cause <code>Throwable</code>, but
    * provides no message text.  The default error number is used.
    *
    * @param   rootCause
    *          Root cause exception/error.
    */
   public NumberedException(Throwable rootCause)
   {
      super(rootCause);
   }
   
   /**
    * Get the error number associated with this exception.
    *
    * @return  Error number (<code>0</code> if never explicitly set to a
    *          different value at construction).
    */
   public int getNumber()
   {
      return numbers == null ? 0 : numbers[numbers.length - 1];
   }
   
   /**
    * Determines if the prefix should be displayed for the error message.
    *
    * @return <code>true</code> if the prefix should be displayed.
    */
   public boolean isPrefix()
   {
      return prefix;
   }
 
   /**
    * Get the errors numbers containing - current error number and previous
    *
    * @return  array of error numbers (list with one element <code>0</code> if never explicitly 
    *          set to a different value at construction).
    */
   public int[] getNumbers()
   {
      return numbers;
   }
}